function prepare_post_effect_rendering()
    love.graphics.setShader()
    love.graphics.setCanvas(canvas)
    love.graphics.clear()
end

function finish_post_effect_rendering()
    love.graphics.setCanvas(canvas_stage_2)
    love.graphics.setShader(shader)
    love.graphics.draw(canvas, 0,0,0,4,4)

    love.graphics.setCanvas()
    love.graphics.setShader(shader_stage_2)
    love.graphics.draw(canvas_stage_2, 0,0,0,1,1)
end

function init_post_effects()
    canvas = love.graphics.newCanvas(love.window.getWidth()/4, love.window.getHeight()/4)
    canvas_stage_2 = love.graphics.newCanvas(love.window.getWidth(),love.window.getHeight())
    shader_stage_2 = love.graphics.newShader [[
        extern number time;
        extern bool enable_distortion;
        
        float scale(float x, float f) {
            return x;
            x = x * 2.0 - 1.0;
            return (x / sqrt(1+x*x) + f*x) * 0.5 + 0.5;
        }

        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords) {
            float offx = 0.0;

            if(enable_distortion) {
                offx = sign(texture_coords.y-fract(0.2*time))*min(max(0.0, 1.0-abs(20.0*(texture_coords.y-fract(0.2*time)))),1.0);
                offx = offx*offx*offx*offx*offx;
            }
            return texture2D(texture, vec2(scale(texture_coords.x, 0.3)+offx, scale(texture_coords.y,0.3)));
        }
    ]]
    shader_stage_2:send('enable_distortion', true)

    shader = love.graphics.newShader [[
        extern vec2 size;
        extern number time;

        float rand(vec2 co)
        {
            // stolen!
            co.x += fract(time/size.x*1.1);
            co.y += fract(time/size.y*1.1);
            float a = 12.9898;
            float b = 78.233;
            float c = 43758.5453;
            float dt= dot(fract(co.xy) ,vec2(a,b));
            float sn= mod(dt,3.14);
            return fract(sin(sn) * c);
        }

        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords) {
            vec2 tc = (floor(texture_coords*size))/size;
            
            vec2 pd = fract(texture_coords-tc)*size-vec2(0.5,0.5);
            float pdd = 1.0-dot(pd,pd);

            vec2 off = vec2(1.0)/size;
            vec3 clr = vec3(0.0);
            tc += vec2(0.5,0.5)/size;
            clr +=  1.0 * texture2D(texture, tc).xyz;
            clr +=  0.1 * texture2D(texture, tc + vec2(0, off.y)).xyz;
            clr +=  0.1 * texture2D(texture, tc + vec2(0, -off.y)).xyz;
            clr +=  0.1 * texture2D(texture, tc + vec2(off.x, 0)).xyz;
            clr +=  0.1 * texture2D(texture, tc + vec2(-off.x, 0)).xyz;
            return  vec4(pdd*mix(clr, vec3(rand(tc)), 0.07), 1.0);
        }
    ]]
    shader:send("size", {love.window.getWidth()/4,love.window.getHeight()/4})
end


